package com.example.controller.house;

import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.pojo.dto.RentInContractStatDto;
import com.example.pojo.vo.RentInContractStatVo;
import com.example.service.house.HouseLeaseInConService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

@Tag(name = "房屋租入")
@Slf4j
@Validated
@RequestMapping("/house-car/house/dist/api/WzHouseLeaseInCon")
@RestController
public class HouseLeaseInConController {
    private final HouseLeaseInConService houseLeaseInConService;

    public HouseLeaseInConController(HouseLeaseInConService houseLeaseInConService) {
        this.houseLeaseInConService = houseLeaseInConService;
    }

    @Operation(summary = "查询租入合同统计")
    @PostMapping("/echartList")
    public Rsp<PageVo<RentInContractStatVo>> rentInContractStat(@Valid @RequestBody RentInContractStatDto dto) {
        return houseLeaseInConService.rentInContractStat(dto);
    }

    @Operation(summary = "租入合同统计导出")
    @GetMapping("/rentInContractStatExport")
    public void rentInContractStatExport(@Valid RentInContractStatDto.RentInContractStatMapDTO dto,
                                         HttpServletResponse response) {
        houseLeaseInConService.rentInContractStatExport(dto, response);
    }

    @GetMapping("/excel")
    public void excel(Integer statisticalMonth, @NotBlank String name, String city, String county,
                      HttpServletResponse response) {
        houseLeaseInConService.excel(statisticalMonth, name, city, county, response);
    }
}
