package com.example.controller.house;

import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.pojo.dto.ListSiteNameDto;
import com.example.pojo.dto.ListSiteNumDto;
import com.example.pojo.vo.ListSiteNameVo;
import com.example.pojo.vo.ListSiteNumVo;
import com.example.service.house.HouseOtnAreaService;
import com.example.utils.R;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

@Tag(name = "不动产组织机构")
@Slf4j
@RequestMapping("/house-car/house/dist/api/wzOtnArea")
@RestController
public class HouseOtnAreaController {
    private final HouseOtnAreaService houseOtnAreaService;

    public HouseOtnAreaController(HouseOtnAreaService houseOtnAreaService) {
        this.houseOtnAreaService = houseOtnAreaService;
    }

    @Operation(summary = "根据父级组织机构id查询子组织机构列表")
    @PostMapping("/getCityOption/{id}")
    public R getCityOption(@PathVariable("id") String id) {
        return houseOtnAreaService.getCityOption(id);
    }

    @Operation(summary = "查询局址别名列表")
    @PostMapping("/listSiteName")
    public Rsp<PageVo<ListSiteNameVo>> listSiteName(@Valid @RequestBody ListSiteNameDto dto) {
        return houseOtnAreaService.listSiteName(dto);
    }

    @Operation(summary = "查询局址编号列表")
    @PostMapping("/listSiteNum")
    public Rsp<PageVo<ListSiteNumVo>> listSiteNum(@Valid @RequestBody ListSiteNumDto dto) {
        return houseOtnAreaService.listSiteNum(dto);
    }
}
