package com.example.pojo.dto;

import com.example.enums.ListRentInCompareOrderEnum;
import com.example.enums.OrderEnum;
import com.example.utils.Page;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;

@Slf4j
@NoArgsConstructor
@Data
public class ListRentInCompareDto {
    @Valid
    @NotNull
    private Page page;
    @Valid
    private ListRentInCompareMapDTO map;

    @Data
    public static class ListRentInCompareMapDTO {
        @Schema(description = "账期", example = "202312")
        private Integer statisticalMonth;
        @Schema(description = "二级组织机构id")
        private String city;
        @Schema(description = "三级级组织机构id")
        private String county;
        @Schema(description = "排序方式", example = "desc")
        private OrderEnum order;
        @Schema(description = "排序字段", example = "totalMonthlyRent")
        private String sidx;

        public ListRentInCompareOrderEnum getOrderColumn() {
            try {
                return ListRentInCompareOrderEnum.valueOf(sidx);
            } catch (Exception e) {
                log.warn(e.getMessage());
                return ListRentInCompareOrderEnum.totalMonthlyRent;
            }
        }
    }

    public ListRentInCompareDto(ListRentInCompareMapDTO map) {
        if (StringUtils.hasText(map.city)) {
            map.city = URLDecoder.decode(map.city, StandardCharsets.UTF_8);
        }
        if (StringUtils.hasText(map.county)) {
            map.county = URLDecoder.decode(map.county, StandardCharsets.UTF_8);
        }
        this.map = map;
    }
}
