package com.example.pojo.po.house;

import lombok.Data;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

@Data
public class RentInContractStatPo {
    /**
     * 二级组织机构编码
     */
    private String areaNo;
    /**
     * 二级组织机构名称
     */
    private String areaName;
    /**
     * 三级组织机构编码
     */
    private String cityNo;
    /**
     * 三级组织机构名称
     */
    private String cityName;
    /**
     * 租入合同数量
     */
    private Integer total;
    /**
     * 合同年金额
     */
    private BigDecimal grossAmountYear;
    /**
     * 每平米单价
     */
    private BigDecimal unitPriceAvg;
    /**
     * 营业面积
     */
    private BigDecimal yy;
    /**
     * 办公面积
     */
    private BigDecimal bg;
    /**
     * 通信机房面积
     */
    private BigDecimal tx;
    /**
     * 其他用途面积
     */
    private BigDecimal qt;
    /**
     * 总计面积
     */
    private BigDecimal zj;
    /**
     * 子列表
     */
    List<RentInContractStatPo> children;

    public RentInContractStatPo() {
        this.areaNo = "";
        this.areaName = "";
        this.cityNo = "";
        this.cityName = "";
        this.total = 0;
        this.grossAmountYear = BigDecimal.ZERO;
        this.unitPriceAvg = BigDecimal.ZERO;
        this.yy = BigDecimal.ZERO;
        this.bg = BigDecimal.ZERO;
        this.tx = BigDecimal.ZERO;
        this.qt = BigDecimal.ZERO;
        this.zj = BigDecimal.ZERO;
        this.children = new ArrayList<>();
    }

    public RentInContractStatPo(String areaNo, String areaName) {
        this.areaNo = areaNo;
        this.areaName = areaName;
        this.cityNo = areaNo;
        this.cityName = areaName;
        this.total = 0;
        this.grossAmountYear = BigDecimal.ZERO;
        this.unitPriceAvg = BigDecimal.ZERO;
        this.yy = BigDecimal.ZERO;
        this.bg = BigDecimal.ZERO;
        this.tx = BigDecimal.ZERO;
        this.qt = BigDecimal.ZERO;
        this.zj = BigDecimal.ZERO;
        this.children = new ArrayList<>();
    }

    public void update(RentInContractStatPo po) {
        this.total = this.total + po.total;
        this.grossAmountYear = this.grossAmountYear.add(po.grossAmountYear);
        this.yy = this.yy.add(po.yy);
        this.bg = this.bg.add(po.bg);
        this.tx = this.tx.add(po.tx);
        this.qt = this.qt.add(po.qt);
        this.zj = this.zj.add(po.zj);
        this.children.add(po);
        this.unitPriceAvg = BigDecimal.valueOf(this.children.stream()
                .mapToDouble(t -> t.getUnitPriceAvg().doubleValue()).average().orElse(0))
                .setScale(2, RoundingMode.HALF_UP);
    }
}
