package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.house.HouseRentInDao;
import com.example.enums.ListRentInContractOrderEnum;
import com.example.enums.OrderEnum;
import com.example.pojo.bo.ListRentInContractBo;
import com.example.pojo.dto.ListRentInContractDto;
import com.example.pojo.po.house.RentInMonthPo;
import com.example.pojo.vo.ListRentInContractVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseContractInService {
    private final HouseRentInDao houseRentInDao;

    public HouseContractInService(HouseRentInDao houseRentInDao) {
        this.houseRentInDao = houseRentInDao;
    }

    public Rsp<PageVo<ListRentInContractVo>> listRentInContract(ListRentInContractDto dto) {
        Page<RentInMonthPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<RentInMonthPo> list = getRentInMonthPos(dto, page);
        List<ListRentInContractVo> vos = new ArrayList<>();
        for (RentInMonthPo po : list) {
            vos.add(new ListRentInContractVo(po));
        }
        PageVo<ListRentInContractVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    private List<RentInMonthPo> getRentInMonthPos(ListRentInContractDto dto, Page<RentInMonthPo> page) {
        ListRentInContractBo bo = new ListRentInContractBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getOrderColumn() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getOrderColumn().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getOrderColumn().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.desc(ListRentInContractOrderEnum.contractAmountIncludingTax.getColumnName()));
        }
        return houseRentInDao.listRentInContract(page, bo);
    }

    public void listRentInContractExport(ListRentInContractDto.ListRentInContractMapDTO dto,
                                         HttpServletResponse response) {
        String filename = "房屋租入_租入合同记录"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeListRentInContract(new ListRentInContractDto(dto), wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void writeListRentInContract(ListRentInContractDto dto, SXSSFWorkbook wb) {
        Page<RentInMonthPo> page = new Page<>(1, Long.MAX_VALUE);
        List<RentInMonthPo> list = getRentInMonthPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 日期样式
        CellStyle dateCellStyle = wb.createCellStyle();
        dateCellStyle.setDataFormat(dataFormat.getFormat("yyyy-MM-dd"));
        dateCellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("房屋所有权人属性", "租入房屋名称", "租入形式", "标准地址(一级)",
                "标准地址(二级)", "标准地址(三级)", "街/路/村门牌号", "城市区域", "地段", "地址经度坐标", "地址纬度坐标",
                "租入建筑面积(平米)", "租入使用面积(平米)", "投资主体", "使用单位层级", "使用单位隶属的省级公司",
                "使用单位隶属的地市级公司", "使用单位隶属的区县级公司", "使用专业线", "租入实际用途", "租入其他用途说明",
                "自用建筑面积(平米)", "转租建筑面积(平米)", "首次租用时间", "联系电话", "合同编号", "合同名称", "合同类型",
                "签订时间", "我方签约主体", "出租方名称", "合同总金额(含税)(元)", "增值税金额(元)", "租入开始时间(合同生效时间)",
                "租入终止时间(合同终止时间)", "承办部门").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 4;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (RentInMonthPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 房屋所有权人属性
            Cell ownerTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getOwnerType() != null) {
                ownerTypeCell.setCellValue(po.getOwnerType());
            }
            // 租入房屋名称
            Cell houseNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getHouseName() != null) {
                houseNameCell.setCellValue(po.getHouseName());
            }
            // 租入形式
            Cell rentTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRentType() != null) {
                rentTypeCell.setCellValue(po.getRentType());
            }
            // 标准地址(一级)
            Cell firstAddressCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFirstAddress() != null) {
                firstAddressCell.setCellValue(po.getFirstAddress());
            }
            // 标准地址(二级)
            Cell secondAddressCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSecondAddress() != null) {
                secondAddressCell.setCellValue(po.getSecondAddress());
            }
            // 标准地址(三级)
            Cell thirdAddressCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getThirdAddress() != null) {
                thirdAddressCell.setCellValue(po.getThirdAddress());
            }
            // 街/路/村门牌号
            Cell fourthAddressCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFourthAddress() != null) {
                fourthAddressCell.setCellValue(po.getFourthAddress());
            }
            // 城市区域
            Cell cityRegionCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityRegion() != null) {
                cityRegionCell.setCellValue(po.getCityRegion());
            }
            // 地段
            Cell areaSectorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaSector() != null) {
                areaSectorCell.setCellValue(po.getAreaSector());
            }
            // 地址经度坐标
            Cell lngCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLng() != null) {
                lngCell.setCellValue(po.getLng().toString());
            }
            // 地址纬度坐标
            Cell latCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLat() != null) {
                latCell.setCellValue(po.getLat().toString());
            }
            // 租入建筑面积(平米)
            Cell buildingAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingArea() != null) {
                buildingAreaCell.setCellValue(po.getBuildingArea().doubleValue());
            }
            buildingAreaCell.setCellStyle(numberCellStyle);
            // 租入使用面积(平米)
            Cell usableAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUsableArea() != null) {
                usableAreaCell.setCellValue(po.getUsableArea().doubleValue());
            }
            usableAreaCell.setCellStyle(numberCellStyle);
            // 投资主体
            Cell investorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getInvestor() != null) {
                investorCell.setCellValue(po.getInvestor());
            }
            // 使用单位层级
            Cell unitLevelCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUnitLevel() != null) {
                unitLevelCell.setCellValue(po.getUnitLevel());
            }
            // 使用单位隶属的省级公司
            Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFirstUnit() != null) {
                firstUnitCell.setCellValue(po.getFirstUnit());
            }
            // 使用单位隶属的地市级公司
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 使用单位隶属的区县级公司
            Cell cityNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityName() != null) {
                cityNameCell.setCellValue(po.getCityName());
            }
            // 使用专业线
            Cell fieldCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getField() != null) {
                fieldCell.setCellValue(po.getField());
            }
            // 租入实际用途
            Cell useTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUseType() != null) {
                useTypeCell.setCellValue(po.getUseType());
            }
            // 租入其他用途说明
            Cell useDescriptionCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUseDescription() != null) {
                useDescriptionCell.setCellValue(po.getUseDescription());
            }
            // 自用建筑面积
            Cell buildingAreaSelfUseCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaSelfUse() != null) {
                buildingAreaSelfUseCell.setCellValue(po.getBuildingAreaSelfUse().doubleValue());
            }
            buildingAreaSelfUseCell.setCellStyle(numberCellStyle);
            // 转租建筑面积
            Cell buildingAreaSubletCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingAreaSublet() != null) {
                buildingAreaSubletCell.setCellValue(po.getBuildingAreaSublet().doubleValue());
            }
            buildingAreaSubletCell.setCellStyle(numberCellStyle);
            // 首次租用时间
            Cell wholeBuildingCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFirstRentDate() != null) {
                wholeBuildingCell.setCellValue(po.getFirstRentDate());
            }
            // 联系电话
            Cell phoneCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getPhone() != null) {
                phoneCell.setCellValue(po.getPhone());
            }
            // 合同编号
            Cell contractNoCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContractNo() != null) {
                contractNoCell.setCellValue(po.getContractNo());
            }
            // 合同名称
            Cell contractNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContractName() != null) {
                contractNameCell.setCellValue(po.getContractName());
            }
            // 合同类型
            Cell contractTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContractType() != null) {
                contractTypeCell.setCellValue(po.getContractType());
            }
            // 签订时间
            Cell signDateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getSignDate() != null) {
                signDateCell.setCellValue(po.getSignDate());
            }
            signDateCell.setCellStyle(dateCellStyle);
            // 我方签约主体
            Cell lesseeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLessee() != null) {
                lesseeCell.setCellValue(po.getLessee());
            }
            // 出租方名称
            Cell lessorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getLessor() != null) {
                lessorCell.setCellValue(po.getLessor());
            }
            // 合同总金额
            Cell grossAmountCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getGrossAmount() != null) {
                grossAmountCell.setCellValue(po.getGrossAmount().doubleValue());
            }
            grossAmountCell.setCellStyle(numberCellStyle);
            // 增值税金额
            Cell vatCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getVat() != null) {
                vatCell.setCellValue(po.getVat().doubleValue());
            }
            vatCell.setCellStyle(numberCellStyle);
            // 租入开始时间
            Cell rentStartDateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRentStartDate() != null) {
                rentStartDateCell.setCellValue(po.getRentStartDate());
            }
            rentStartDateCell.setCellStyle(dateCellStyle);
            // 租入终止时间
            Cell rentEndDateCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRentEndDate() != null) {
                rentEndDateCell.setCellValue(po.getRentEndDate());
            }
            rentEndDateCell.setCellStyle(dateCellStyle);
            // 承办部门
            Cell undertakingDepartment = row.createCell(columnIndex.getAndIncrement());
            if (po.getUndertakingDepartment() != null) {
                undertakingDepartment.setCellValue(po.getUndertakingDepartment());
            }
        }
    }
}
