package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.house.HouseRentInDao;
import com.example.enums.ListRentInTopOrderEnum;
import com.example.enums.OrderEnum;
import com.example.pojo.bo.ListRentInTopBo;
import com.example.pojo.dto.ListRentInTopDto;
import com.example.pojo.po.house.RentInMonthPo;
import com.example.pojo.vo.ListRentInTopVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseContractTopService {
    private final HouseRentInDao houseRentInDao;

    public HouseContractTopService(HouseRentInDao houseRentInDao) {
        this.houseRentInDao = houseRentInDao;
    }

    public Rsp<PageVo<ListRentInTopVo>> listRentInTop(ListRentInTopDto dto) {
        Page<RentInMonthPo> page = new Page<>(dto.getPage().getPageNum(), dto.getPage().getPageSize());
        List<RentInMonthPo> list = getRentInMonthPos(dto, page);
        List<ListRentInTopVo> vos = new ArrayList<>();
        for (RentInMonthPo po : list) {
            vos.add(new ListRentInTopVo(po));
        }
        PageVo<ListRentInTopVo> pageVo = new PageVo<>(vos, page);
        return Rsp.ok(pageVo);
    }

    private List<RentInMonthPo> getRentInMonthPos(ListRentInTopDto dto, Page<RentInMonthPo> page) {
        ListRentInTopBo bo = new ListRentInTopBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        if (dto.getMap() != null && dto.getMap().getOrder() != null && dto.getMap().getOrderColumn() != null) {
            if (OrderEnum.asc.equals(dto.getMap().getOrder())) {
                page.addOrder(OrderItem.asc(dto.getMap().getOrderColumn().getColumnName()));
            } else {
                page.addOrder(OrderItem.desc(dto.getMap().getOrderColumn().getColumnName()));
            }
        } else {
            page.addOrder(OrderItem.desc(ListRentInTopOrderEnum.totalMonthlyRent.getColumnName()));
        }
        return houseRentInDao.listRentInTop(page, bo);
    }

    public void listRentInTopExport(ListRentInTopDto.ListRentInTopMapDTO dto, HttpServletResponse response) {
        String filename = "房屋租入_租入房屋单价前10名"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeListRentInTop(new ListRentInTopDto(dto), wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void writeListRentInTop(ListRentInTopDto dto, SXSSFWorkbook wb) {
        Page<RentInMonthPo> page = new Page<>(1, Long.MAX_VALUE);
        List<RentInMonthPo> list = getRentInMonthPos(dto, page);
        DataFormat dataFormat = wb.createDataFormat();
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        SXSSFSheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("使用单位隶属的省级公司", "使用单位隶属的地市级公司",
                "使用单位隶属的区县级公司", "使用专业线", "租入实际用途", "租入其他用途说明", "城市区域", "地段",
                "合同总金额(含税)(元)", "租入建筑面积(平米)", "租期(年)", "每平米月租金", "合同编号", "合同名称").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256 * 2;
            sheet.setColumnWidth(i, columnWidth);
        }
        // 数据
        for (RentInMonthPo po : list) {
            AtomicInteger columnIndex = new AtomicInteger(0);
            Row row = sheet.createRow(rowIndex.getAndIncrement());
            // 使用单位隶属的省级公司
            Cell firstUnitCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getFirstUnit() != null) {
                firstUnitCell.setCellValue(po.getFirstUnit());
            }
            // 使用单位隶属的地市级公司
            Cell areaNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaName() != null) {
                areaNameCell.setCellValue(po.getAreaName());
            }
            // 使用单位隶属的区县级公司
            Cell cityNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityName() != null) {
                cityNameCell.setCellValue(po.getCityName());
            }
            // 使用专业线
            Cell fieldCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getField() != null) {
                fieldCell.setCellValue(po.getField());
            }
            // 租入实际用途
            Cell useTypeCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUseType() != null) {
                useTypeCell.setCellValue(po.getUseType());
            }
            // 租入其他用途说明
            Cell useDescriptionCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUseDescription() != null) {
                useDescriptionCell.setCellValue(po.getUseDescription());
            }
            // 城市区域
            Cell cityRegionCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getCityRegion() != null) {
                cityRegionCell.setCellValue(po.getCityRegion());
            }
            // 地段
            Cell areaSectorCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getAreaSector() != null) {
                areaSectorCell.setCellValue(po.getAreaSector());
            }
            // 合同总金额
            Cell grossAmountCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getGrossAmount() != null) {
                grossAmountCell.setCellValue(po.getGrossAmount().doubleValue());
            }
            grossAmountCell.setCellStyle(numberCellStyle);
            // 租入建筑面积(平米)
            Cell buildingAreaCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getBuildingArea() != null) {
                buildingAreaCell.setCellValue(po.getBuildingArea().doubleValue());
            }
            buildingAreaCell.setCellStyle(numberCellStyle);
            // 租期
            Cell zqCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getRentYears() != null) {
                zqCell.setCellValue(po.getRentYears().doubleValue());
            }
            zqCell.setCellStyle(numberCellStyle);
            // 每平米月租金
            Cell unitPriceCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getUnitPrice2() != null) {
                unitPriceCell.setCellValue(po.getUnitPrice2().doubleValue());
            }
            unitPriceCell.setCellStyle(numberCellStyle);
            // 合同编号
            Cell contractNoCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContractNo() != null) {
                contractNoCell.setCellValue(po.getContractNo());
            }
            // 合同名称
            Cell contractNameCell = row.createCell(columnIndex.getAndIncrement());
            if (po.getContractName() != null) {
                contractNameCell.setCellValue(po.getContractName());
            }
        }
    }
}
