package com.example.service.house;

import cn.hutool.core.net.URLEncodeUtil;
import com.example.common.PageVo;
import com.example.common.Rsp;
import com.example.dao.house.HouseRentInDao;
import com.example.pojo.bo.RentInContractStatBo;
import com.example.pojo.dto.ListRentInCompareDto;
import com.example.pojo.dto.ListRentInContractDto;
import com.example.pojo.dto.ListRentInTopDto;
import com.example.pojo.dto.RentInContractStatDto;
import com.example.pojo.dto.RentInUseStatDto;
import com.example.pojo.po.house.RentInContractStatPo;
import com.example.pojo.vo.RentInContractStatVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Stream;

@Slf4j
@Service
public class HouseLeaseInConService {
    private final HouseRentInDao houseRentInDao;
    @Autowired
    private HouseContractInService houseContractInService;
    @Autowired
    private HouseUseService houseUseService;
    @Autowired
    private HouseContractTopService houseContractTopService;
    @Autowired
    private HouseContractCompareService houseContractCompareService;

    public HouseLeaseInConService(HouseRentInDao houseRentInDao) {
        this.houseRentInDao = houseRentInDao;
    }

    public Rsp<PageVo<RentInContractStatVo>> rentInContractStat(RentInContractStatDto dto) {
        RentInContractStatPo pos = getRentInContractStatPos(dto);
        if (pos == null) {
            return Rsp.ok();
        }
        RentInContractStatVo vo = new RentInContractStatVo(pos, false);
        PageVo<RentInContractStatVo> pageVo = new PageVo<>(Collections.singletonList(vo));
        return Rsp.ok(pageVo);
    }

    private RentInContractStatPo getRentInContractStatPos(RentInContractStatDto dto) {
        RentInContractStatBo bo = new RentInContractStatBo();
        if (dto.getMap() != null && dto.getMap().getStatisticalMonth() != null) {
            bo.setYearMonth(dto.getMap().getStatisticalMonth());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCity()) && !"全省".equals(dto.getMap().getCity())) {
            bo.setAreaNo(dto.getMap().getCity());
        }
        if (dto.getMap() != null && StringUtils.hasText(dto.getMap().getCounty())) {
            bo.setCityNo(dto.getMap().getCounty());
        }
        // 查询三级单位统计
        List<RentInContractStatPo> pos = houseRentInDao.rentInContractStat(bo);
        if (CollectionUtils.isEmpty(pos)) {
            return null;
        }
        // 一级单位统计
        RentInContractStatPo firstUnitPo = new RentInContractStatPo();
        firstUnitPo.setAreaName("全省");
        firstUnitPo.setCityName("全省");
        LinkedHashMap<String, RentInContractStatPo> secondUnitPoMap = new LinkedHashMap<>();
        // 二级单位统计
        for (RentInContractStatPo houseAgeStatPo : pos) {
            secondUnitPoMap.putIfAbsent(houseAgeStatPo.getAreaNo(),
                    new RentInContractStatPo(houseAgeStatPo.getAreaNo(), houseAgeStatPo.getAreaName()));
            RentInContractStatPo secondUnitPo = secondUnitPoMap.get(houseAgeStatPo.getAreaNo());
            secondUnitPo.update(houseAgeStatPo);
        }
        for (RentInContractStatPo value : secondUnitPoMap.values()) {
            firstUnitPo.update(value);
        }
        if (firstUnitPo.getTotal().equals(0)) {
            return null;
        }
        return firstUnitPo;
    }

    public void rentInContractStatExport(RentInContractStatDto.RentInContractStatMapDTO dto,
                                         HttpServletResponse response) {
        String filename = "房屋租入_租入合同统计"
                + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss")) + ".xlsx";
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncodeUtil.encode(filename));
        response.setContentType("application/octet-stream");
        try (SXSSFWorkbook wb = new SXSSFWorkbook()) {
            writeRentInContractStat(new RentInContractStatDto(dto), wb);
            wb.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private void writeRentInContractStat(RentInContractStatDto dto, SXSSFWorkbook wb) {
        RentInContractStatPo firstPo = getRentInContractStatPos(dto);
        DataFormat dataFormat = wb.createDataFormat();
        // 整数样式
        CellStyle integerCellStyle = wb.createCellStyle();
        integerCellStyle.setDataFormat(dataFormat.getFormat("#,##0"));
        integerCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 数字样式
        CellStyle numberCellStyle = wb.createCellStyle();
        numberCellStyle.setDataFormat(dataFormat.getFormat("#,##0.00"));
        numberCellStyle.setAlignment(HorizontalAlignment.RIGHT);
        // 默认样式
        CellStyle baseCellStyle = wb.createCellStyle();
        baseCellStyle.setAlignment(HorizontalAlignment.CENTER);
        baseCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
        Sheet sheet = wb.createSheet();
        AtomicInteger rowIndex = new AtomicInteger(0);
        // 表头
        Row headerRow = sheet.createRow(rowIndex.getAndIncrement());
        List<String> headers = Stream.of("资产所属单位(一级)", "资产所属单位(二级)", "资产所属单位(三级)",
                "租入合同数量", "合同年金额(含税)(元)", "每平米单价(元)","营业面积(平米)", "办公面积(平米)", "通信机房面积(平米)",
                "其他用途面积(平米)", "总计面积(平米)").toList();
        int headerLength = headers.size();
        for (int i = 0; i < headerLength; i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellValue(headers.get(i));
            cell.setCellStyle(baseCellStyle);
            // 根据内容长度设置列宽
            int columnWidth = headers.get(i).length() * 256 * 2 + 256;
            sheet.setColumnWidth(i, columnWidth);
        }
        if (firstPo == null) {
            return;
        }
        // 数据
        AtomicInteger columnIndex1 = new AtomicInteger(0);
        // 一级单位统计
        Row row1 = sheet.createRow(rowIndex.getAndIncrement());
        // 资产所属单位(一级)
        Cell firstUnitCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getAreaName() != null) {
            firstUnitCell1.setCellValue(firstPo.getAreaName());
        }
        // 二级单位
        Cell areaNameCell1 = row1.createCell(columnIndex1.getAndIncrement());
//        if (firstPo.getAreaName() != null) {
//            areaNameCell1.setCellValue(firstPo.getAreaName());
//        }
        // 三级单位
        Cell cityNameCell1 = row1.createCell(columnIndex1.getAndIncrement());
//        if (firstPo.getCityName() != null) {
//            cityNameCell1.setCellValue(firstPo.getCityName());
//        }
        // 租入合同数量
        Cell totalCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getTotal() != null) {
            totalCell1.setCellValue(firstPo.getTotal());
        }
        totalCell1.setCellStyle(integerCellStyle);
        // 合同年金额
        Cell grossAmountYearCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getGrossAmountYear() != null) {
            grossAmountYearCell1.setCellValue(firstPo.getGrossAmountYear().doubleValue());
        }
        grossAmountYearCell1.setCellStyle(numberCellStyle);
        // 每平米单价
        Cell unitPriceAvgCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getUnitPriceAvg() != null) {
            unitPriceAvgCell1.setCellValue(firstPo.getUnitPriceAvg().doubleValue());
        }
        unitPriceAvgCell1.setCellStyle(numberCellStyle);
        // 营业面积
        Cell yyCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getYy() != null) {
            yyCell1.setCellValue(firstPo.getYy().doubleValue());
        }
        yyCell1.setCellStyle(numberCellStyle);
        // 办公面积
        Cell bgCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getBg() != null) {
            bgCell1.setCellValue(firstPo.getBg().doubleValue());
        }
        bgCell1.setCellStyle(numberCellStyle);
        // 通信机房面积
        Cell txCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getTx() != null) {
            txCell1.setCellValue(firstPo.getTx().doubleValue());
        }
        txCell1.setCellStyle(numberCellStyle);
        // 其他用途面积
        Cell qtCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getQt() != null) {
            qtCell1.setCellValue(firstPo.getQt().doubleValue());
        }
        qtCell1.setCellStyle(numberCellStyle);
        // 总计面积
        Cell zjCell1 = row1.createCell(columnIndex1.getAndIncrement());
        if (firstPo.getZj() != null) {
            zjCell1.setCellValue(firstPo.getZj().doubleValue());
        }
        zjCell1.setCellStyle(numberCellStyle);
        // 二级单位统计
        for (RentInContractStatPo secondPo : firstPo.getChildren()) {
            AtomicInteger columnIndex2 = new AtomicInteger(0);
            // 一级单位统计
            Row row2 = sheet.createRow(rowIndex.getAndIncrement());
            // 资产所属单位(一级)
            Cell firstUnitCell2 = row2.createCell(columnIndex2.getAndIncrement());
//            if (secondPo.getAreaName() != null) {
//                firstUnitCell2.setCellValue(secondPo.getAreaName());
//            }
            // 二级单位
            Cell areaNameCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getAreaName() != null) {
                areaNameCell2.setCellValue(secondPo.getAreaName());
            }
            // 三级单位
            Cell cityNameCell2 = row2.createCell(columnIndex2.getAndIncrement());
//            if (secondPo.getCityName() != null) {
//                cityNameCell2.setCellValue(secondPo.getCityName());
//            }
            // 租入合同数量
            Cell totalCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getTotal() != null) {
                totalCell2.setCellValue(secondPo.getTotal());
            }
            totalCell2.setCellStyle(integerCellStyle);
            // 合同年金额
            Cell grossAmountYearCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getGrossAmountYear() != null) {
                grossAmountYearCell2.setCellValue(secondPo.getGrossAmountYear().doubleValue());
            }
            grossAmountYearCell2.setCellStyle(numberCellStyle);
            // 每平米单价
            Cell unitPriceAvgCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getUnitPriceAvg() != null) {
                unitPriceAvgCell2.setCellValue(secondPo.getUnitPriceAvg().doubleValue());
            }
            unitPriceAvgCell2.setCellStyle(numberCellStyle);
            // 营业面积
            Cell yyCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getYy() != null) {
                yyCell2.setCellValue(secondPo.getYy().doubleValue());
            }
            yyCell2.setCellStyle(numberCellStyle);
            // 办公面积
            Cell bgCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getBg() != null) {
                bgCell2.setCellValue(secondPo.getBg().doubleValue());
            }
            bgCell2.setCellStyle(numberCellStyle);
            // 通信机房面积
            Cell txCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getTx() != null) {
                txCell2.setCellValue(secondPo.getTx().doubleValue());
            }
            txCell2.setCellStyle(numberCellStyle);
            // 其他用途面积
            Cell qtCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getQt() != null) {
                qtCell2.setCellValue(secondPo.getQt().doubleValue());
            }
            qtCell2.setCellStyle(numberCellStyle);
            // 总计面积
            Cell zjCell2 = row2.createCell(columnIndex2.getAndIncrement());
            if (secondPo.getZj() != null) {
                zjCell2.setCellValue(secondPo.getZj().doubleValue());
            }
            zjCell2.setCellStyle(numberCellStyle);
            // 三级单位统计
            for (RentInContractStatPo thirdPo : secondPo.getChildren()) {
                AtomicInteger columnIndex3 = new AtomicInteger(0);
                Row row3 = sheet.createRow(rowIndex.getAndIncrement());
                // 资产所属单位(一级)
                Cell firstUnitCell3 = row3.createCell(columnIndex3.getAndIncrement());
//                if (thirdPo.getAreaName() != null) {
//                    firstUnitCell3.setCellValue(thirdPo.getAreaName());
//                }
                // 二级单位
                Cell areaNameCell3 = row3.createCell(columnIndex3.getAndIncrement());
//                if (thirdPo.getAreaName() != null) {
//                    areaNameCell3.setCellValue(thirdPo.getAreaName());
//                }
                // 三级单位
                Cell cityNameCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getCityName() != null) {
                    cityNameCell3.setCellValue(thirdPo.getCityName());
                }
                // 租入合同数量
                Cell totalCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getTotal() != null) {
                    totalCell3.setCellValue(thirdPo.getTotal());
                }
                totalCell3.setCellStyle(integerCellStyle);
                // 合同年金额
                Cell grossAmountYearCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getGrossAmountYear() != null) {
                    grossAmountYearCell3.setCellValue(thirdPo.getGrossAmountYear().doubleValue());
                }
                grossAmountYearCell3.setCellStyle(numberCellStyle);
                // 每平米单价
                Cell unitPriceAvgCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getUnitPriceAvg() != null) {
                    unitPriceAvgCell3.setCellValue(thirdPo.getUnitPriceAvg().doubleValue());
                }
                unitPriceAvgCell3.setCellStyle(numberCellStyle);
                // 营业面积
                Cell yyCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getYy() != null) {
                    yyCell3.setCellValue(thirdPo.getYy().doubleValue());
                }
                yyCell3.setCellStyle(numberCellStyle);
                // 办公面积
                Cell bgCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getBg() != null) {
                    bgCell3.setCellValue(thirdPo.getBg().doubleValue());
                }
                bgCell3.setCellStyle(numberCellStyle);
                // 通信机房面积
                Cell txCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getTx() != null) {
                    txCell3.setCellValue(thirdPo.getTx().doubleValue());
                }
                txCell3.setCellStyle(numberCellStyle);
                // 其他用途面积
                Cell qtCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getQt() != null) {
                    qtCell3.setCellValue(thirdPo.getQt().doubleValue());
                }
                qtCell3.setCellStyle(numberCellStyle);
                // 总计面积
                Cell zjCell3 = row3.createCell(columnIndex3.getAndIncrement());
                if (thirdPo.getZj() != null) {
                    zjCell3.setCellValue(thirdPo.getZj().doubleValue());
                }
                zjCell3.setCellStyle(numberCellStyle);
            }
        }
    }

    public void excel(Integer statisticalMonth, String name, String city, String county, HttpServletResponse response) {
        name = URLDecoder.decode(name, StandardCharsets.UTF_8);
        if ("合同管理".equals(name)) {
            RentInContractStatDto.RentInContractStatMapDTO dto = new RentInContractStatDto.RentInContractStatMapDTO();
            dto.setStatisticalMonth(statisticalMonth);
            dto.setCity(city);
            dto.setCounty(county);
            rentInContractStatExport(dto, response);
        } else if ("租入合同记录".equals(name)) {
            ListRentInContractDto.ListRentInContractMapDTO dto = new ListRentInContractDto.ListRentInContractMapDTO();
            dto.setStatisticalMonth(statisticalMonth);
            dto.setCity(city);
            dto.setCounty(county);
            houseContractInService.listRentInContractExport(dto, response);
        } else if ("租入用途管理".equals(name)) {
            RentInUseStatDto.RentInUseStatMapDTO dto = new RentInUseStatDto.RentInUseStatMapDTO();
            dto.setStatisticalMonth(statisticalMonth);
            dto.setCity(city);
            dto.setCounty(county);
            houseUseService.rentInUseStatExport(dto, response);
        } else if ("租入房屋单价前10名".equals(name)) {
            ListRentInTopDto.ListRentInTopMapDTO dto = new ListRentInTopDto.ListRentInTopMapDTO();
            dto.setStatisticalMonth(statisticalMonth);
            dto.setCity(city);
            dto.setCounty(county);
            houseContractTopService.listRentInTopExport(dto, response);
        } else if ("租入单价与平均单价比较".equals(name)) {
            ListRentInCompareDto.ListRentInCompareMapDTO dto = new ListRentInCompareDto.ListRentInCompareMapDTO();
            dto.setStatisticalMonth(statisticalMonth);
            dto.setCity(city);
            dto.setCounty(county);
            houseContractCompareService.listRentInCompareExport(dto, response);
        }
    }
}
